part of sliders;

/// Base class for [SfRangeSlider] and [SfRangeSelector] track shapes.
abstract class SfTrackShape {
  const SfTrackShape();

  /// Returns the size based on the values passed to it.
  Rect getPreferredRect(
      RenderBox parentBox, SfSliderThemeData themeData, Offset offset,
      {bool isActive}) {
    final double maxRadius = math.max(themeData.overlayRadius,
        math.max(themeData.thumbRadius, themeData.tickSize.width / 2));
    final double left = offset.dx + maxRadius;
    double top = offset.dy;
    if (isActive != null) {
      final double maxTrackHeight =
          math.max(themeData.activeTrackHeight, themeData.inactiveTrackHeight);
      top += isActive
          ? (maxTrackHeight - themeData.activeTrackHeight) / 2
          : (maxTrackHeight - themeData.inactiveTrackHeight) / 2;
    }
    final double width = parentBox.size.width - 2 * maxRadius;
    final double height = isActive == null
        ? math.max(themeData.activeTrackHeight, themeData.inactiveTrackHeight)
        : (isActive
            ? themeData.activeTrackHeight
            : themeData.inactiveTrackHeight);
    return Rect.fromLTWH(left, top, width, height);
  }

  /// Paints the track based on the values passed to it.
  void paint(
    PaintingContext context,
    Offset offset,
    Offset thumbCenter,
    Offset startThumbCenter,
    Offset endThumbCenter, {
    RenderBox parentBox,
    SfSliderThemeData themeData,
    SfRangeValues currentValues,
    dynamic currentValue,
    Animation<double> enableAnimation,
    Paint inactivePaint,
    Paint activePaint,
    TextDirection textDirection,
  }) {
    final Radius radius = Radius.circular(themeData.trackCornerRadius);
    final Rect inactiveTrackRect =
        getPreferredRect(parentBox, themeData, offset, isActive: false);
    final Rect activeTrackRect =
        getPreferredRect(parentBox, themeData, offset, isActive: true);

    if (inactivePaint == null) {
      inactivePaint = Paint();
      final ColorTween inactiveTrackColorTween = ColorTween(
          begin: themeData.disabledInactiveTrackColor,
          end: themeData.inactiveTrackColor);
      inactivePaint.color = inactiveTrackColorTween.evaluate(enableAnimation);
    }

    if (activePaint == null) {
      activePaint = Paint();
      final ColorTween activeTrackColorTween = ColorTween(
          begin: themeData.disabledActiveTrackColor,
          end: themeData.activeTrackColor);
      activePaint.color = activeTrackColorTween.evaluate(enableAnimation);
    }

    if (textDirection == TextDirection.rtl) {
      if (thumbCenter == null) {
        final Offset temp = startThumbCenter;
        startThumbCenter = endThumbCenter;
        endThumbCenter = temp;
      } else {
        final Paint temp = activePaint;
        activePaint = inactivePaint;
        inactivePaint = temp;
      }
    }

    RRect inactiveTrackRRect;
    if (thumbCenter == null) {
      // Drawing inactive track.
      Rect trackRect = Rect.fromLTRB(inactiveTrackRect.left,
          inactiveTrackRect.top, startThumbCenter.dx, inactiveTrackRect.bottom);
      inactiveTrackRRect = RRect.fromRectAndCorners(trackRect,
          topLeft: radius, bottomLeft: radius);
      context.canvas.drawRRect(inactiveTrackRRect, inactivePaint);

      // Drawing active track.
      final Rect activeTrackRRect = Rect.fromLTRB(startThumbCenter.dx,
          activeTrackRect.top, endThumbCenter.dx, activeTrackRect.bottom);
      context.canvas.drawRect(activeTrackRRect, activePaint);

      // Drawing inactive track.
      trackRect = Rect.fromLTRB(
          endThumbCenter.dx,
          inactiveTrackRect.top,
          inactiveTrackRect.width + inactiveTrackRect.left,
          inactiveTrackRect.bottom);
      inactiveTrackRRect = RRect.fromRectAndCorners(trackRect,
          topLeft: Radius.zero,
          topRight: radius,
          bottomLeft: Radius.zero,
          bottomRight: radius);
      context.canvas.drawRRect(inactiveTrackRRect, inactivePaint);
    } else {
      // Drawing active track.
      Rect trackRect = Rect.fromLTRB(activeTrackRect.left, activeTrackRect.top,
          thumbCenter.dx, activeTrackRect.bottom);
      final RRect activeTrackRRect = RRect.fromRectAndCorners(trackRect,
          topLeft: radius, bottomLeft: radius);
      context.canvas.drawRRect(activeTrackRRect, activePaint);

      // Drawing inactive track.
      trackRect = Rect.fromLTRB(
          thumbCenter.dx,
          inactiveTrackRect.top,
          inactiveTrackRect.width + inactiveTrackRect.left,
          inactiveTrackRect.bottom);
      inactiveTrackRRect = RRect.fromRectAndCorners(trackRect,
          topLeft: Radius.zero,
          topRight: radius,
          bottomLeft: Radius.zero,
          bottomRight: radius);
      context.canvas.drawRRect(inactiveTrackRRect, inactivePaint);
    }
  }
}

/// Base class for [SfRangeSlider] and [SfRangeSelector] thumb shapes.
abstract class SfThumbShape {
  const SfThumbShape();

  /// Returns the size based on the values passed to it.
  Size getPreferredSize(SfSliderThemeData themeData) {
    return Size.fromRadius(themeData.thumbRadius);
  }

  /// Paints the thumb based on the values passed to it.
  void paint(PaintingContext context, Offset center,
      {RenderBox parentBox,
      RenderBox child,
      SfSliderThemeData themeData,
      SfRangeValues currentValues,
      dynamic currentValue,
      Paint paint,
      Animation<double> enableAnimation,
      TextDirection textDirection,
      SfThumb thumb}) {
    final double radius = getPreferredSize(themeData).width / 2;

    if (paint == null) {
      paint = Paint();
      paint.isAntiAlias = true;
      paint.color = ColorTween(
              begin: themeData.disabledThumbColor, end: themeData.thumbColor)
          .evaluate(enableAnimation);
    }

    context.canvas
        .drawCircle(center, getPreferredSize(themeData).width / 2, paint);
    if (child != null) {
      context.paintChild(
          child,
          Offset(center.dx - (child.size.width) / 2,
              center.dy - (child.size.height) / 2));
    }

    if (themeData.thumbStrokeColor != null &&
        themeData.thumbStrokeWidth != null &&
        themeData.thumbStrokeWidth > 0) {
      context.canvas.drawCircle(
          center,
          themeData.thumbStrokeWidth > radius
              ? radius / 2
              : radius - themeData.thumbStrokeWidth / 2,
          paint
            ..color = themeData.thumbStrokeColor
            ..style = PaintingStyle.stroke
            ..strokeWidth = themeData.thumbStrokeWidth > radius
                ? radius
                : themeData.thumbStrokeWidth);
    }
  }
}

/// Base class for [SfRangeSlider] and [SfRangeSelector] divisors shapes.
abstract class SfDivisorShape {
  const SfDivisorShape();

  /// Returns the size based on the values passed to it.
  Size getPreferredSize(SfSliderThemeData themeData, {bool isActive}) {
    return Size.fromRadius(isActive != null
        ? (isActive
            ? themeData.activeDivisorRadius ?? themeData.activeTrackHeight / 2
            : themeData.inactiveDivisorRadius ??
                themeData.inactiveTrackHeight / 2)
        : 0);
  }

  /// Paints the divisors based on the values passed to it.
  void paint(PaintingContext context, Offset center, Offset thumbCenter,
      Offset startThumbCenter, Offset endThumbCenter,
      {RenderBox parentBox,
      SfSliderThemeData themeData,
      SfRangeValues currentValues,
      dynamic currentValue,
      Paint paint,
      Animation<double> enableAnimation,
      TextDirection textDirection}) {
    bool isActive;
    switch (textDirection) {
      case TextDirection.ltr:
        // Added this condition to check whether consider single thumb or two thumbs for finding active range.
        isActive = startThumbCenter != null
            ? center.dx >= startThumbCenter.dx && center.dx <= endThumbCenter.dx
            : center.dx <= thumbCenter.dx;
        break;
      case TextDirection.rtl:
        isActive = startThumbCenter != null
            ? center.dx >= endThumbCenter.dx && center.dx <= startThumbCenter.dx
            : center.dx >= thumbCenter.dx;
        break;
    }

    if (paint == null) {
      paint = Paint();
      final Color begin = isActive
          ? themeData.disabledActiveDivisorColor
          : themeData.disabledInactiveDivisorColor;
      final Color end = isActive
          ? themeData.activeDivisorColor
          : themeData.inactiveDivisorColor;

      paint.color =
          ColorTween(begin: begin, end: end).evaluate(enableAnimation);
    }

    final double divisorRadius =
        getPreferredSize(themeData, isActive: isActive).width / 2;
    context.canvas.drawCircle(center, divisorRadius, paint);

    final double divisorStrokeWidth = isActive
        ? themeData.inactiveDivisorStrokeWidth
        : themeData.activeDivisorStrokeWidth;
    final Color divisorStrokeColor = isActive
        ? themeData.inactiveDivisorStrokeColor
        : themeData.activeDivisorStrokeColor;

    if (divisorStrokeColor != null &&
        divisorStrokeWidth != null &&
        divisorStrokeWidth > 0) {
      // Drawing divisor stroke
      context.canvas.drawCircle(
          center,
          divisorStrokeWidth > divisorRadius
              ? divisorRadius / 2
              : divisorRadius - divisorStrokeWidth / 2,
          paint
            ..color = divisorStrokeColor
            ..style = PaintingStyle.stroke
            ..strokeWidth = divisorStrokeWidth > divisorRadius
                ? divisorRadius
                : divisorStrokeWidth);
    }
  }
}

/// Base class for [SfRangeSlider] and [SfRangeSelector] overlay shapes.
abstract class SfOverlayShape {
  const SfOverlayShape();

  /// Returns the size based on the values passed to it.
  Size getPreferredSize(SfSliderThemeData themeData) {
    return Size.fromRadius(themeData.overlayRadius);
  }

  /// Paints the overlay based on the values passed to it.
  void paint(PaintingContext context, Offset center,
      {RenderBox parentBox,
      SfSliderThemeData themeData,
      SfRangeValues currentValues,
      dynamic currentValue,
      Paint paint,
      Animation<double> animation,
      SfThumb thumb}) {
    final double radius = getPreferredSize(themeData).width / 2;
    final Tween<double> tween = Tween<double>(begin: 0.0, end: radius);

    if (paint == null) {
      paint = Paint();
      paint.color = themeData.overlayColor;
    }
    context.canvas.drawCircle(center, tween.evaluate(animation), paint);
  }
}

/// Base class for [SfRangeSlider] and [SfRangeSelector] major tick shapes.
abstract class SfTickShape {
  const SfTickShape();

  /// Returns the size based on the values passed to it.
  Size getPreferredSize(SfSliderThemeData themeData) {
    return Size.copy(themeData.tickSize);
  }

  /// Paints the major ticks based on the values passed to it.
  void paint(PaintingContext context, Offset offset, Offset thumbCenter,
      Offset startThumbCenter, Offset endThumbCenter,
      {RenderBox parentBox,
      SfSliderThemeData themeData,
      SfRangeValues currentValues,
      dynamic currentValue,
      Animation<double> enableAnimation,
      TextDirection textDirection}) {
    bool isInactive;
    final Size tickSize = getPreferredSize(themeData);
    switch (textDirection) {
      case TextDirection.ltr:
        // Added this condition to check whether consider single thumb or two thumbs for finding inactive range.
        isInactive = startThumbCenter != null
            ? offset.dx < startThumbCenter.dx || offset.dx > endThumbCenter.dx
            : offset.dx > thumbCenter.dx;
        break;
      case TextDirection.rtl:
        isInactive = startThumbCenter != null
            ? offset.dx > startThumbCenter.dx || offset.dx < endThumbCenter.dx
            : offset.dx < thumbCenter.dx;
        break;
    }

    final Color begin = isInactive
        ? themeData.disabledInactiveTickColor
        : themeData.disabledActiveTickColor;
    final Color end =
        isInactive ? themeData.inactiveTickColor : themeData.activeTickColor;
    final Paint paint = Paint()
      ..isAntiAlias = true
      ..strokeWidth = tickSize.width
      ..color = ColorTween(begin: begin, end: end).evaluate(enableAnimation);
    context.canvas.drawLine(
        offset, Offset(offset.dx, offset.dy + tickSize.height), paint);
  }
}

/// A base class which is used to render rectangular or paddle shape tooltip.
abstract class SfTooltipShape {
  const SfTooltipShape();

  /// Draws the tooltip based on the values passed in the arguments.
  void paint(
    PaintingContext context,
    Offset thumbCenter,
    Offset offset,
    TextPainter textPainter, {
    RenderBox parentBox,
    SfSliderThemeData sliderThemeData,
    Paint paint,
    Animation<double> animation,
    Rect trackRect,
  });
}

class _SfTrackShape extends SfTrackShape {}

class _SfThumbShape extends SfThumbShape {}

class _SfDivisorShape extends SfDivisorShape {}

class _SfOverlayShape extends SfOverlayShape {}

class _SfTickShape extends SfTickShape {}

/// A class which is used to render paddle shape tooltip.
class SfPaddleTooltipShape extends SfTooltipShape {
  /// Draws the tooltip based on the values passed in the arguments.
  @override
  void paint(
    PaintingContext context,
    Offset thumbCenter,
    Offset offset,
    TextPainter textPainter, {
    RenderBox parentBox,
    SfSliderThemeData sliderThemeData,
    Paint paint,
    Animation<double> animation,
    Rect trackRect,
  }) {
    final double paddleScaleFactor =
        animation.value * textPainter.height / _defaultTextSize;
    final double inverseTextScale = animation.value *
        (paddleScaleFactor != 0 ? 1.0 / paddleScaleFactor : 0.0);
    final double textHalfWidth = textPainter.width / 2.0;

    context.canvas.save();
    context.canvas.translate(thumbCenter.dx, thumbCenter.dy - offset.dy);
    context.canvas.scale(paddleScaleFactor, paddleScaleFactor);

    final double rightBottomNeckEndAngle = math.pi +
        (60 * math.pi / 180) +
        math.atan(-_bottomNeckOffsetY / _bottomNeckOffsetX);

    final double paddleCircleHalfWidth = math.max(
      0.0,
      inverseTextScale * textHalfWidth - (_paddleCircleRadius - _textPadding),
    );

    final Rect paddleCircleRect = Rect.fromLTWH(
      -_paddleCircleRadius - paddleCircleHalfWidth,
      -_paddleCircleRadius - _paddleCircleCenterY,
      2.0 * (_paddleCircleRadius + paddleCircleHalfWidth),
      2.0 * _paddleCircleRadius,
    );

    final Offset paddleLeftPosition =
        (paddleCircleRect.topLeft * paddleScaleFactor) + thumbCenter;
    final Offset paddleRightPosition =
        (paddleCircleRect.bottomRight * paddleScaleFactor) + thumbCenter;
    double shiftPaddleCircleWidth = 0.0;

    if (paddleLeftPosition.dx < trackRect.left) {
      shiftPaddleCircleWidth = trackRect.left - paddleLeftPosition.dx;
    }

    if (paddleRightPosition.dx > trackRect.right) {
      shiftPaddleCircleWidth = trackRect.right - paddleRightPosition.dx;
    }

    shiftPaddleCircleWidth = paddleScaleFactor == 0.0
        ? 0.0
        : shiftPaddleCircleWidth / paddleScaleFactor;
    if (shiftPaddleCircleWidth < 0.0) {
      shiftPaddleCircleWidth =
          math.max(shiftPaddleCircleWidth, -paddleCircleHalfWidth);
    } else {
      shiftPaddleCircleWidth =
          math.min(shiftPaddleCircleWidth, paddleCircleHalfWidth);
    }

    final double leftShiftWidth =
        paddleCircleHalfWidth - shiftPaddleCircleWidth;
    final double rightShiftWidth =
        paddleCircleHalfWidth + shiftPaddleCircleWidth;

    final double leftShiftPosition =
        math.max(0.0, math.min(1.0, leftShiftWidth / _topNeckOffsetX));
    final double rightShiftPosition =
        math.max(0.0, math.min(1.0, rightShiftWidth / _topNeckOffsetX));
    final double leftShiftDegree =
        (1.0 - leftShiftPosition) * _thirtyDegreesRadian;
    final double rightShiftDegree =
        (1.0 - rightShiftPosition) * _thirtyDegreesRadian;

    final Offset leftTopNeckOffset = Offset(
      -_topNeckOffsetX,
      _paddleCircleCenter.dy + math.cos(leftShiftDegree) * _paddleCircleCenterY,
    );
    final Offset rightTopNeckOffset = Offset(
      _topNeckOffsetX,
      _paddleCircleCenter.dy +
          math.cos(rightShiftDegree) * _paddleCircleCenterY,
    );

    final double leftTopNeckDegree = _ninetyDegreesRadian - leftShiftDegree;
    final double rightTopNeckDegree =
        _twoSeventyDegreesRadian - rightShiftDegree;

    final Path path = Path();
    path.arcTo(
        Rect.fromCircle(center: leftTopNeckOffset, radius: _topNeckRadius),
        0.0,
        -leftTopNeckDegree,
        false);

    path.arcTo(
        Rect.fromCircle(
            center: _paddleCircleCenter - Offset(leftShiftWidth, 0.0),
            radius: _paddleCircleRadius),
        _ninetyDegreesRadian + leftShiftDegree,
        math.pi - leftShiftDegree,
        false);

    path.arcTo(
        Rect.fromCircle(
            center: _paddleCircleCenter + Offset(rightShiftWidth, 0.0),
            radius: _paddleCircleRadius),
        _twoSeventyDegreesRadian,
        math.pi - rightShiftDegree,
        false);

    path.arcTo(
        Rect.fromCircle(center: rightTopNeckOffset, radius: _topNeckRadius),
        rightTopNeckDegree,
        math.pi - rightTopNeckDegree,
        false);

    path.arcTo(
        Rect.fromCircle(
            center: const Offset(_bottomNeckOffsetX, 0.0),
            radius: _bottomNeckRadius),
        math.pi,
        -math.pi + rightBottomNeckEndAngle,
        false);

    path.arcTo(
        Rect.fromCircle(
            center: const Offset(-_bottomNeckOffsetX, 0.0),
            radius: _bottomNeckRadius),
        math.pi - rightBottomNeckEndAngle,
        0.0,
        false);

    context.canvas.drawPath(path, paint);
    context.canvas.save();
    context.canvas.translate(shiftPaddleCircleWidth, -_paddleCircleCenterY);
    context.canvas.scale(inverseTextScale, inverseTextScale);
    textPainter.paint(context.canvas,
        Offset.zero - Offset(textHalfWidth, textPainter.height / 2.0));
    context.canvas.restore();
    context.canvas.restore();
  }
}

/// A class which is used to render rectangular shape tooltip.
class SfRectangularTooltipShape extends SfTooltipShape {
  /// Draws the tooltip based on the values passed in the arguments.
  @override
  void paint(PaintingContext context, Offset thumbCenter, Offset offset,
      TextPainter textPainter,
      {RenderBox parentBox,
      SfSliderThemeData sliderThemeData,
      Paint paint,
      Animation<double> animation,
      Rect trackRect}) {
    final double leftPadding = _tooltipTextPadding.dx / 2;
    final double minLeftdx = trackRect.left;

    context.canvas.save();
    context.canvas.translate(thumbCenter.dx, thumbCenter.dy);
    context.canvas.scale(animation.value);
    final Path path = getTooltipPath(textPainter.size + _tooltipTextPadding,
        offset.dy, trackRect, thumbCenter.dx);
    context.canvas
        .drawPath(path, paint..color = sliderThemeData.tooltipBackgroundColor);

    final double dx = _getActualTooltipTextX(minLeftdx, thumbCenter,
        textPainter.width / 2, leftPadding, trackRect, textPainter, path);
    final double dy = offset.dy +
        _tooltipTriangleHeight +
        (path.getBounds().size.height - _tooltipTriangleHeight) / 2 +
        textPainter.height / 2;
    textPainter.paint(context.canvas, Offset(dx, -dy));
    context.canvas.restore();
  }

  double _getActualTooltipTextX(
      double minLeftdx,
      Offset thumbCenter,
      double halfTextWidth,
      double leftPadding,
      Rect trackRect,
      TextPainter textPainter,
      Path path) {
    final double dx = thumbCenter.dx - path.getBounds().size.width / 2;
    return dx >= minLeftdx
        ? thumbCenter.dx + halfTextWidth + leftPadding > trackRect.right
            ? -halfTextWidth -
                path.getBounds().size.width / 2 +
                trackRect.right -
                thumbCenter.dx
            : -halfTextWidth
        : -halfTextWidth +
            path.getBounds().size.width / 2 +
            trackRect.left -
            thumbCenter.dx;
  }

  Path getTooltipPath(
      Size textSize, double tooltipStartY, Rect trackRect, double dx) {
    final double dy = tooltipStartY + _tooltipTriangleHeight;
    final double tooltipWidth =
        textSize.width < _minTooltipWidth ? _minTooltipWidth : textSize.width;
    final double tooltipHeight = textSize.height < _minTooltipHeight
        ? _minTooltipHeight
        : textSize.height;
    final double halfTooltipWidth = tooltipWidth / 2;

    double rightLineWidth = dx + halfTooltipWidth > trackRect.right
        ? trackRect.right - dx
        : halfTooltipWidth;
    final double leftLineWidth = dx - halfTooltipWidth < trackRect.left
        ? dx - trackRect.left
        : tooltipWidth - rightLineWidth;
    rightLineWidth = leftLineWidth < halfTooltipWidth
        ? halfTooltipWidth - leftLineWidth + rightLineWidth
        : rightLineWidth;
    const double halfTooltipTriangleWidth = _tooltipTriangleWidth / 2;

    final Path path = Path();
    path.moveTo(0, -tooltipStartY);
    //    /
    final bool canAdjustTooltipNose =
        rightLineWidth > halfTooltipTriangleWidth + _cornerRadius / 2;
    path.lineTo(
        canAdjustTooltipNose ? halfTooltipTriangleWidth : rightLineWidth,
        -dy - (canAdjustTooltipNose ? 0 : _cornerRadius / 2));
    //      ___
    //     /
    path.lineTo(rightLineWidth - _cornerRadius, -dy);
    //      ___|
    //     /
    path.quadraticBezierTo(
        rightLineWidth, -dy, rightLineWidth, -dy - _cornerRadius);
    path.lineTo(rightLineWidth, -dy - tooltipHeight + _cornerRadius);
    //  _______
    //      ___|
    //     /
    path.quadraticBezierTo(rightLineWidth, -dy - tooltipHeight,
        rightLineWidth - _cornerRadius, -dy - tooltipHeight);
    path.lineTo(-leftLineWidth + _cornerRadius, -dy - tooltipHeight);
    //  _______
    // |    ___|
    //     /
    path.quadraticBezierTo(-leftLineWidth, -dy - tooltipHeight, -leftLineWidth,
        -dy - tooltipHeight + _cornerRadius);
    path.lineTo(-leftLineWidth, -dy - _cornerRadius);
    //  ________
    // |___  ___|
    //      /
    if (leftLineWidth > halfTooltipTriangleWidth) {
      path.quadraticBezierTo(
          -leftLineWidth, -dy, -leftLineWidth + _cornerRadius, -dy);
      path.lineTo(-halfTooltipTriangleWidth, -dy);
    }
    //  ________
    // |___  ___|
    //     \/
    path.close();
    return path;
  }
}

class _SfMinorTickShape extends SfTickShape {
  @override
  Size getPreferredSize(SfSliderThemeData themeData) {
    return Size.copy(themeData.minorTickSize);
  }

  @override
  void paint(PaintingContext context, Offset offset, Offset thumbCenter,
      Offset startThumbCenter, Offset endThumbCenter,
      {RenderBox parentBox,
      SfRangeValues currentValues,
      dynamic currentValue,
      SfSliderThemeData themeData,
      Animation<double> enableAnimation,
      TextDirection textDirection}) {
    bool isInactive;
    final Size minorTickSize = getPreferredSize(themeData);
    switch (textDirection) {
      case TextDirection.ltr:
        isInactive = startThumbCenter != null
            ? offset.dx < startThumbCenter.dx || offset.dx > endThumbCenter.dx
            : offset.dx > thumbCenter.dx;
        break;
      case TextDirection.rtl:
        isInactive = startThumbCenter != null
            ? offset.dx > startThumbCenter.dx || offset.dx < endThumbCenter.dx
            : offset.dx < thumbCenter.dx;
        break;
    }

    final Color begin = isInactive
        ? themeData.disabledInactiveMinorTickColor
        : themeData.disabledActiveMinorTickColor;
    final Color end = isInactive
        ? themeData.inactiveMinorTickColor
        : themeData.activeMinorTickColor;
    final Paint paint = Paint()
      ..isAntiAlias = true
      ..strokeWidth = minorTickSize.width
      ..color = ColorTween(begin: begin, end: end).evaluate(enableAnimation);
    context.canvas.drawLine(
        offset, Offset(offset.dx, offset.dy + minorTickSize.height), paint);
  }
}
